#include <windows.h>

#include "Bitmap.h"
#include "AudioSystem.h"
#include <cmath>

HINSTANCE g_hInst;
HWND g_hWnd;

Bitmap bmp;

AudioSystem audio;

void ResizeBmp()
{
	RECT rc;
	GetClientRect(g_hWnd,&rc);
	bmp.resize(rc.right-rc.left,rc.bottom-rc.top);
	bmp.SetFont( Font(28,"Calibri",false) );
	bmp.SetTextcolor(0);
	InvalidateRect(g_hWnd,&rc,FALSE);
}
void Redraw()
{
	RECT rc;
	GetClientRect(g_hWnd,&rc);
	InvalidateRect(g_hWnd,&rc,FALSE);
}

void Render()
{
	bmp.cls(0xFFFFFF);
	bmp.RenderCenteredText(bmp.getWidth()/2, bmp.getHeight()/2 - 15, "Hint: Try bashing on the keyboard.");
}


/*
Treat keyboard as a piano sorta.
 2 3   5 6 7   9 0   =
q w e r t y u i o p [ ] \
C D E F G A B C D E F G A

 s d   g h j   l ;
z x c v b n m , . /
C D E F G A B C D E
*/

const char* notekeys = "2356790=qwertyuiop[]\\sdghjl;zxcvbnm,./";
const char noteoctave[] = {
  4,4,4,4,4,5,5,5,
  4,4,4,4,4,4,4,5,5,5,5,5,5,
  3,3,3,3,3,4,4,
  3,3,3,3,3,3,3,4,4,4
};
const char note[] = {
   1,3,  6,8,10,   1,3,  6,
  0,2,4,5,7,9, 11,0,2,4,5,7,9,
   1,3,  6,8,10,   1,3, 
  0,2,4,5,7,9, 11,0,2,4
};



void PlaySound(int octave, int note)
{
	// Calculate frequency!
	int MiddleA = 4*12 + 9;
	int ThisNote = octave*12 + note;
	int semitone_shift = ThisNote - MiddleA;
	float frequency = 440 * pow(2.0f,(float)semitone_shift/12.0f);

	audio.PlaySnd( new SimpleSound(frequency, 1.0f, 0.0f) );
}

void PlaySoundForKey(int key)
{
	int i = 0;
	while(notekeys[i] != 0)
	{
		if(notekeys[i] == key)
		{
			PlaySound(noteoctave[i],note[i]);
			return;
		}
		i++;
	}
}


LRESULT WINAPI WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
	case WM_CREATE:
		g_hWnd = hWnd;
		ResizeBmp();
		audio.Start();
		return 0;
	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;
	case WM_SIZE:
		switch(wParam)
		{
		case SIZE_MAXHIDE:
		case SIZE_MAXSHOW:
		case SIZE_MAXIMIZED:
		case SIZE_RESTORED:
			ResizeBmp();
		}
		return 0;
	case WM_PAINT:
		{
			HDC hDC;
			PAINTSTRUCT ps;
			hDC = BeginPaint(hWnd,&ps);

			Render();
			BitBlt(hDC,0,0,bmp.getWidth(),bmp.getHeight(),bmp.DC(),0,0,SRCCOPY);

			EndPaint(hWnd,&ps);

		}
		return 0;
	case WM_CHAR:
		PlaySoundForKey(wParam);
		return 0;

	}
	return DefWindowProc(hWnd,msg,wParam,lParam);
}






int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR cmdline, int nsc)
{
	g_hInst = hInst;
	g_hWnd = 0;

	WNDCLASS wc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=0;
	wc.hbrBackground=0;
	wc.hCursor=LoadCursor(NULL,IDC_ARROW);
	wc.hIcon=0;
	wc.hInstance=hInst;
	wc.lpfnWndProc=&WndProc;
	wc.lpszClassName="testcls";
	wc.lpszMenuName=0;
	wc.style=CS_HREDRAW|CS_VREDRAW;
	RegisterClass(&wc);


	HWND hWnd = CreateWindow("testcls","audio_lib_test",WS_OVERLAPPEDWINDOW,CW_USEDEFAULT,CW_USEDEFAULT,500,500,0,0,hInst,0);

	if(!hWnd)
	{
		MessageBox(0,"Error creating window","gfx_lib_test",MB_ICONERROR);
		return 1;
	}

	ShowWindow(hWnd,SW_SHOW);

	MSG msg;
	do {
		GetMessage(&msg,0,0,0);
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	} while(msg.message != WM_QUIT);

	return 0;
}