#include "Bitmap.h"

//class Color
//{
//public:
Color::Color( unsigned long rgbcolor )
{
	c = rgbcolor;
}
Color::Color( int r, int g, int b )
{
	if(r>255) r=255; if(g>255) g=255; if(b>255) b=255;
	if(r<0) r=0; if(g<0) g=0; if(b<0) b=0;
	c = (r<<16) | (g<<8) | (b);
}

int Color::red()
{
	return (c>>16) & 255;
}
int Color::blue()
{
	return (c) & 255;
}
int Color::green()
{
	return (c >> 8) & 255;
}

COLORREF Color::colorref()
{
	return (c&0xFF00FF00) | ((c>>16)&0xFF) | ((c<<16)&0xFF0000);
}


//	unsigned long c;
//};

//class Font
//{
//public:
Font::Font()
{
	font_ref=0;
	font=0;
	gotrequestedfont=true;
	TrySetFont(10,true,NULL);
}
Font::Font( const Font & f )
{
	CopyFont(f);
}
Font::Font(int height, char * facename, bool fixedwidth)
{
	font_ref = 0;
	gotrequestedfont = true;
	if(!TrySetFont(height,fixedwidth,facename))
	{
		gotrequestedfont=false;
		TrySetFont(height,fixedwidth,NULL);
	}
}
Font::~Font()
{
	KillFont();
	font_ref=0;
	font=0;
}
Font & Font::operator= (const Font & f)
{
	if(this==&f) return *this;
	KillFont();
	CopyFont(f);
	return *this;
}

bool Font::GotRequestedFont()
{
	return gotrequestedfont;
}

HFONT Font::GetFont()
{
	return font;
}

//protected:
bool Font::TrySetFont( int height, bool fixedwidth, char* facename )
{
	KillFont();

	font = CreateFont( height, 0, 0, 0, 0, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
		DEFAULT_QUALITY, fixedwidth?FIXED_PITCH:DEFAULT_PITCH, facename);

	if(!font) return false;
	font_ref = new int;
	font_ref[0] = 1;
	return true;
}
void Font::CopyFont( const Font & f )
{
	font = f.font;
	gotrequestedfont = f.gotrequestedfont;
	font_ref = f.font_ref;
	font_ref[0]++;
}

void Font::KillFont()
{
	if(!font_ref) return;
	font_ref[0]--;
	if(!font_ref[0])
	{
		delete font_ref;
		DeleteObject(font);
	}
	font_ref=0;
	font=0;
}
//	HFONT font;
//	int * font_ref;
//	bool gotrequestedfont;
//};


//class Bitmap
//{
//public:
Bitmap::Bitmap(void)
{
	ConstructBitmap(16,16);
}
Bitmap::Bitmap(int width, int height)
{
	ConstructBitmap(width,height);
}
Bitmap::Bitmap( const Bitmap & b )
{
	ConstructBitmap(b.wid, b.hei);
	memcpy(bmpbits,b.bmpbits,wid*hei*4);
}
Bitmap::~Bitmap(void)
{
	DestroyBitmap(hDC,hBmp);
	hDC=0;
	hBmp=0;
}
Bitmap & Bitmap::operator=( const Bitmap & b )
{
	if(this==&b) return *this;
	DestroyBitmap(hDC,hBmp);
	ConstructBitmap(b.wid, b.hei);
	memcpy(bmpbits,b.bmpbits,wid*hei*4);
	return *this;
}

int Bitmap::getWidth()
{
	return wid;
}
int Bitmap::getHeight()
{
	return hei;
}
HDC Bitmap::DC()
{
	return hDC;
}

unsigned long * Bitmap::getBits()
{
	return bmpbits;
}


void Bitmap::resize( int w, int h, bool preserve, bool scale)
{
	if(!preserve)
	{
		DestroyBitmap(hDC,hBmp);
		ConstructBitmap(w,h);
	}
	else
	{
		HDC oldhDC;
		HBITMAP oldhBmp;
		int oldwid, oldhei;
		oldwid = wid;
		oldhei = hei;
		oldhDC = hDC;
		oldhBmp=hBmp;
		ConstructBitmap(w,h);
		if(scale)
		{
			StretchBlt(hDC,0,0,wid,hei,oldhDC,0,0,oldwid,oldhei,SRCCOPY);
		}
		else
		{
			BitBlt(hDC,0,0,w,h,oldhDC,0,0,SRCCOPY);
		}
		DestroyBitmap( oldhDC, oldhBmp);
	}
}

void Bitmap::cls( Color fillcolor )
{
	int i;
	for(i=0;i<wid*hei;i++) bmpbits[i] = fillcolor.c;
}

void Bitmap::clearrect( int x1, int x2, int y1, int y2, Color fillcolor )
{
	if(x1<0) x1=0;
	if(x2>=wid) x2=wid-1;
	if(y1<0) y1=0;
	if(y2>=hei) y2=hei-1;
	int x;
	for(;y1<=y2;y1++)
	{
		for(x=x1;x<=x2;x++)
		{
			bmpbits[x+y1*wid]=fillcolor.c;
		}
	}
}

void Bitmap::pset(int x, int y, Color c)
{
	if(x<0 || x>=wid || y<0 || y>=hei) return;
	bmpbits[x+y*wid] = c.c;
}
Color Bitmap::point(int x, int y)
{
	if(x<0 || x>=wid || y<0 || y>=hei) return 0;
	return bmpbits[x+y*wid];
}


void Bitmap::hline( int x1, int x2, int y, Color linecolor )
{
	if(y<0 || y>=hei) return;
	if(x1<0) x1=0;
	if(x2>=wid) x2=wid-1;
	for(;x1<=x2;x1++)
	{
		bmpbits[x1+y*wid]=linecolor.c;
	}
}
void Bitmap::vline( int x, int y1, int y2, Color linecolor )
{
	if(x<0 || x>=hei) return;
	if(y1<0) y1=0;
	if(y2>=hei) y2=hei-1;
	for(;y1<=y2;y1++)
	{
		bmpbits[x+y1*wid]=linecolor.c;
	}
}

void Bitmap::line( int x1, int y1, int x2, int y2, Color linecolor )
{

}

void Bitmap::SetTextcolor( Color textcolor )
{
	SetTextColor(hDC, textcolor.colorref());
}

void Bitmap::SetFont( Font & f )
{
	curfont=f;
	SelectObject(hDC,curfont.GetFont());
}
void Bitmap::RenderText( int x, int y, const char * text, bool rightalign)
{
	RECT rc;
	rc.top = y;
	rc.bottom = y+1000;
	rc.left = rc.right = x;
	if(rightalign)
	{
		rc.left -= 10000;
	}
	else
	{
		rc.right += 10000;
	}
	DrawText(hDC,text,-1,&rc,DT_TOP|(rightalign?DT_RIGHT:DT_LEFT));
}
void Bitmap::RenderCenteredText( int x, int y, const char * text )
{
	RECT rc;
	rc.top = y;
	rc.bottom = y+1000;
	rc.left = rc.right = x;
	rc.left -= 10000;
	rc.right += 10000;
	DrawText(hDC,text,-1,&rc,DT_TOP|DT_CENTER);
}

void Bitmap::RenderTextBlock( int x, int y, int width, int height, const char * text, bool wordbreak)
{
	RECT rc;
	rc.top = y;
	rc.bottom = y+height;
	rc.left = rc.right = x;
	rc.right += width;
	DrawText(hDC,text,-1,&rc,DT_TOP|DT_LEFT|(wordbreak?DT_WORDBREAK:0));
}




void Bitmap::MakeLayeredAlpha()
{
	int i;
	unsigned long px;
	int r,g,b,a;
	for(i=0;i<wid*hei;i++)
	{
		px = bmpbits[i];
		a = (px>>24)&255;	r = (px>>16)&255;	g = (px>>8)&255;	b = (px)&255;		
		r=(r*a)/255;
		g=(g*a)/255;
		b=(b*a)/255;
		bmpbits[i] = (a<<24) | (r<<16) | (g<<8) | (b);
	}
}


//protected:
void Bitmap::ConstructBitmap(int w, int h)
{
	wid = w;
	hei = h;

	BITMAPINFO bi;
	bi.bmiHeader.biBitCount=32;
	bi.bmiHeader.biClrImportant=0;
	bi.bmiHeader.biClrUsed=0;
	bi.bmiHeader.biCompression=BI_RGB;
	bi.bmiHeader.biHeight=-h;
	bi.bmiHeader.biPlanes=1;
	bi.bmiHeader.biSize=sizeof(BITMAPINFOHEADER);
	bi.bmiHeader.biSizeImage=0;
	bi.bmiHeader.biWidth=w;
	bi.bmiHeader.biXPelsPerMeter=0;
	bi.bmiHeader.biYPelsPerMeter=0;

	HDC hDesktopDC = GetDC(GetDesktopWindow());
	hBmp = CreateDIBSection(hDesktopDC,&bi, 0, (void **)&bmpbits, 0, 0);
	hDC = CreateCompatibleDC(hDesktopDC);
	ReleaseDC(GetDesktopWindow(),hDesktopDC);
	SelectObject(hDC,hBmp);

	SelectObject(hDC,curfont.GetFont()); // also select font!

	SetBkMode(hDC,TRANSPARENT);
}
void Bitmap::DestroyBitmap( HDC hKillDC, HBITMAP hKillBitmap)
{
	DeleteDC(hKillDC);
	DeleteObject(hKillBitmap);
}

//	int wid, hei;
//	HDC hDC;
//	HBITMAP hBmp;
//	unsigned long * bmpbits;
