#include <windows.h>

#include "Bitmap.h"
#include "Rasterizer.h"
#include "Transform.h"
#include "BitmapWindow.h"
#include <cmath>
#include <vector>

HINSTANCE g_hInst;
HWND g_hWnd;

HMENU hMenu;
Bitmap bmp;
const int Num_Tests = 3;
int cur_test = -1;

const double Pi = 3.1415926535897932384626433832795;

__int64 teststarttime;
__int64 lasttesttime;




BitmapWindow * bmpwnd;
std::vector<VertexTexture> texvtxs;
std::vector<VertexColor> colorvtxs;

void EnterTest(int index)
{
	if(index<0 || index>=Num_Tests) return;

	{
		WINDOWINFO wi;
		GetWindowInfo(g_hWnd,&wi);
		bmpwnd->bmp.resize(wi.rcClient.right-wi.rcClient.left,wi.rcClient.bottom-wi.rcClient.top);
		bmpwnd->bmp.cls(0);
		bmpwnd->Move(wi.rcClient.left,wi.rcClient.top);
		bmpwnd->Redraw();
	}
	// Cleanup previous test
	switch(cur_test)
	{
	case 0:
		break;
	}
	GetSystemTimeAsFileTime((LPFILETIME)&teststarttime);
	lasttesttime = teststarttime;
	// Setup new test
	switch(index)
	{
	case 0:
		break;
	}
	cur_test = index;
}


void RenderRotatingTriangle(Bitmap & b, double totalseconds)
{
	Rasterizer<false,true> r;

	r.Bind(&b);

	b.cls(0);

	VertexColor v1, v2, v3;

	double cx,cy, rad, sn,cs;
	cx = b.getWidth()/2.0;
	cy = b.getHeight()/2.0;
	rad = 200;
	double theta = totalseconds;
	sn = sin(theta)*rad; cs=cos(theta)*rad;
	v1 = VertexColor((float)(cx+sn),(float)(cy+cs),1,1,0,0,1);
	theta+=Pi*2/3; sn = sin(theta)*rad; cs=cos(theta)*rad;
	v2 = VertexColor((float)(cx+sn),(float)(cy+cs),1,0,1,0,1);
	theta+=Pi*2/3; sn = sin(theta)*rad; cs=cos(theta)*rad;
	v3 = VertexColor((float)(cx+sn),(float)(cy+cs),1,0,0,1,1);

	r.Rasterize<VertexColor,PixelShaderColor,BlendModeOverwrite>(&v1,&v2,&v3);
}
void CreateTextureCube()
{


}

void CreateLightSpot(float x, float y, float rad, float r, float g, float b, int segments=50)
{
	colorvtxs.clear();
	colorvtxs.reserve(segments*3);
	float angle, lastangle, sn, cs, lsn, lcs;
	lastangle = 0;
	int i;
	for(i=0;i<segments;i++)
	{
		angle = (float)(lastangle + 2*Pi/segments);
		sn = sin(angle)*rad; cs = cos(angle)*rad;
		lsn = sin(lastangle)*rad; lcs = cos(lastangle)*rad;

		colorvtxs.push_back(VertexColor(x,y,1,r,g,b,1));
		colorvtxs.push_back(VertexColor(x+sn,y+cs,1,0,0,0,1));
		colorvtxs.push_back(VertexColor(x+lsn,y+lcs,1,0,0,0,1));

		lastangle = angle;
	}
}

void Render()
{
	__int64 curtime;
	double totalseconds;
	double deltaseconds;
	GetSystemTimeAsFileTime((LPFILETIME)&curtime);
	deltaseconds = (curtime-lasttesttime)/ 10000000.0; // 100-nanosecond intervals (10^-7, seven 0s)
	totalseconds = (curtime-teststarttime)/ 10000000.0;
	lasttesttime=curtime;

	switch(cur_test)
	{
	case -1:
		bmp.cls(Color(102,188,192));
		bmp.SetTextcolor(0);
		bmp.RenderCenteredText(bmp.getWidth()/2,bmp.getHeight()/2-15,"Select a test to begin.");
		break;

	case 0: // triangle
		bmp.cls(0);
		RenderRotatingTriangle(bmpwnd->bmp, totalseconds);
		bmpwnd->Redraw();
		break;
	case 1: // triangle as texture
		RenderRotatingTriangle(bmp, totalseconds);
		// render textured cube
		CreateTextureCube();


		break;
	case 2: // additive blending
		bmp.cls(0);
		{
			Rasterizer<false,true> r;
			r.Bind(&bmp);


			CreateLightSpot(bmp.getWidth()/2,bmp.getHeight()/2,200,1,0,0);
			r.RasterizeTriangleList<VertexColor,PixelShaderColor,BlendModeAdditive>(colorvtxs);

			CreateLightSpot(200,200,200,0,1,0);
			r.RasterizeTriangleList<VertexColor,PixelShaderColor,BlendModeAdditive>(colorvtxs);

			CreateLightSpot(300,300,200,0,0,1);
			r.RasterizeTriangleList<VertexColor,PixelShaderColor,BlendModeAdditive>(colorvtxs);

		}



		break;
	}
	/*
	
	*/
}

void ResizeBmp()
{
	RECT rc;
	GetClientRect(g_hWnd,&rc);
	bmp.resize(rc.right-rc.left,rc.bottom-rc.top);
	bmp.SetFont( Font(30,"Calibri",false) );
	InvalidateRect(g_hWnd,&rc,FALSE);
}

bool floating_l_down;
int floating_mx, floating_my;
LRESULT WINAPI FloatingWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam, void *userdata)
{
	short int mx,my;
	mx = LOWORD(lParam);
	my = HIWORD(lParam);
	switch(msg)
	{
	case WM_LBUTTONDOWN:
		SetCapture(hWnd);
		floating_l_down = true;
		floating_mx = mx;
		floating_my = my;
		return 0;
	case WM_LBUTTONUP:
		ReleaseCapture();
		floating_l_down = false;
		return 0;
	case WM_MOUSEMOVE:
		if(floating_l_down)
		{
			int dmx, dmy;
			dmx = mx - floating_mx;
			dmy = my - floating_my;

			bmpwnd->Move(bmpwnd->GetX()+dmx, bmpwnd->GetY()+dmy);
			bmpwnd->Redraw();
		}
		return 0;
	}
	return DefWindowProc(hWnd,msg,wParam,lParam);
}


LRESULT WINAPI WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
	case WM_CREATE:
		g_hWnd = hWnd;
		ResizeBmp();
		return 0;
	case WM_DESTROY:
		PostQuitMessage(0);
		return 0;
	case WM_SIZE:
		switch(wParam)
		{
		case SIZE_MAXHIDE:
		case SIZE_MAXSHOW:
		case SIZE_MAXIMIZED:
		case SIZE_RESTORED:
			ResizeBmp();
		}
		return 0;
	case WM_PAINT:
		{
			HDC hDC;
			PAINTSTRUCT ps;
			hDC = BeginPaint(hWnd,&ps);

			Render();
			BitBlt(hDC,0,0,bmp.getWidth(),bmp.getHeight(),bmp.DC(),0,0,SRCCOPY);

			EndPaint(hWnd,&ps);

			RECT rc;
			GetClientRect(hWnd,&rc);
			InvalidateRect(hWnd,&rc,FALSE);
		}
		return 0;
	case WM_COMMAND:
		switch(HIWORD(wParam))
		{
		case 0:
		case 1:
			{
				int index = LOWORD(wParam);
				if(index >=1000 && index<1000+Num_Tests)
				{
					EnterTest(index-1000);
				}
			}
		}
	}
	return DefWindowProc(hWnd,msg,wParam,lParam);
}






int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR cmdline, int nsc)
{
	g_hInst = hInst;
	g_hWnd = 0;

	WNDCLASS wc;
	wc.cbClsExtra=0;
	wc.cbWndExtra=0;
	wc.hbrBackground=0;
	wc.hCursor=LoadCursor(NULL,IDC_ARROW);
	wc.hIcon=0;
	wc.hInstance=hInst;
	wc.lpfnWndProc=&WndProc;
	wc.lpszClassName="testcls";
	wc.lpszMenuName=0;
	wc.style=CS_HREDRAW|CS_VREDRAW;
	RegisterClass(&wc);

	// Create menu;
	hMenu = CreateMenu();
	HMENU hpop = CreatePopupMenu();
	AppendMenu(hMenu,MF_POPUP,(UINT)hpop,"Pick Test");

	int i;
	for(i=0;i<Num_Tests;i++)
	{
		char temp[128];
		wsprintf(temp,"Test %i",i+1);
		AppendMenu(hpop,MF_STRING,1000+i,temp);
	}

	HWND hWnd = CreateWindow("testcls","gfx_lib_test",WS_OVERLAPPEDWINDOW,CW_USEDEFAULT,CW_USEDEFAULT,500,500,0,hMenu,hInst,0);

	if(!hWnd)
	{
		MessageBox(0,"Error creating window","gfx_lib_test",MB_ICONERROR);
		return 1;
	}

	ShowWindow(hWnd,SW_SHOW);

	bmpwnd = new BitmapWindow(10,10,500,500);
	bmpwnd->SetWndProc(FloatingWndProc,0);

	MSG msg;
	do {
		if(PeekMessage(&msg,0,0,0,PM_REMOVE))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
		{
			RECT rc;
			GetClientRect(hWnd,&rc);
			InvalidateRect(hWnd,&rc,FALSE);
		}
	} while(msg.message != WM_QUIT);

	return 0;
}